@@ -11,11 +11,16 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import socket
+import datetime
+
+import pytest
+import botocore.endpoint
 
 from tests import mock
 from tests import unittest
 
 from botocore.compat import six
+from botocore.config import Config
 from botocore.endpoint import Endpoint, DEFAULT_TIMEOUT
 from botocore.endpoint import EndpointCreator
 from botocore.exceptions import HTTPClientError
@@ -122,6 +127,29 @@ class TestEndpointFeatures(TestEndpointBase):
         request = prepare.call_args[0][0]
         self.assertEqual(request.context['signing']['region'], 'us-west-2')
 
+    def test_make_request_sets_retries_config_in_context(self):
+        r = request_dict()
+        r['context'] = {'signing': {'region': 'us-west-2'}}
+        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+            self.endpoint.make_request(self.op, r)
+        request = prepare.call_args[0][0]
+        self.assertIn('retries', request.context)
+
+    def test_exception_caught_when_constructing_retries_context(self):
+        r = request_dict()
+        datetime_patcher = mock.patch.object(
+            botocore.endpoint.datetime, 'datetime',
+            mock.Mock(wraps=datetime.datetime)
+        )
+        r['context'] = {'signing': {'region': 'us-west-2'}}
+        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+            mocked_datetime = datetime_patcher.start()
+            mocked_datetime.side_effect = Exception()
+            self.endpoint.make_request(self.op, r)
+            datetime_patcher.stop()
+        request = prepare.call_args[0][0]
+        self.assertIn('retries', request.context)
+
     def test_parses_modeled_exception_fields(self):
         # Setup the service model to have exceptions to generate the mapping
         self.service_model = mock.Mock(spec=ServiceModel)
@@ -186,7 +214,9 @@ class TestRetryInterface(TestEndpointBase):
     def test_retry_events_can_alter_behavior(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
             num_retries=1)
-        self.endpoint.make_request(self._operation, request_dict())
+        r = request_dict()
+        r['context']['client_config'] = Config()
+        self.endpoint.make_request(self._operation, r)
         self.assert_events_emitted(
             self.event_emitter,
             expected_events=[
@@ -219,7 +249,9 @@ class TestRetryInterface(TestEndpointBase):
         parser = mock.Mock()
         parser.parse.return_value = {'ResponseMetadata': {}}
         self.factory.return_value.create_parser.return_value = parser
-        response = self.endpoint.make_request(self._operation, request_dict())
+        r = request_dict()
+        r['context']['client_config'] = Config()
+        response = self.endpoint.make_request(self._operation, r)
         self.assertEqual(response[1]['ResponseMetadata']['RetryAttempts'], 1)
 
     def test_retry_attempts_is_zero_when_not_retried(self):
@@ -255,6 +287,7 @@ class TestS3ResetStreamOnRetry(TestEndpointBase):
         op.metadata = {'protocol': 'rest-xml'}
         request = request_dict()
         request['body'] = body
+        request['context']['client_config'] = Config()
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
             num_retries=2
         )
@@ -293,6 +326,25 @@ class TestEndpointCreator(unittest.TestCase):
             endpoint_url='https://endpoint.url')
         self.assertEqual(endpoint.host, 'https://endpoint.url')
 
+    def test_creates_endpoint_with_ipv4_url(self):
+        endpoint = self.creator.create_endpoint(
+            self.service_model, region_name='us-east-1',
+            endpoint_url='https://192.168.0.0')
+        self.assertEqual(endpoint.host, 'https://192.168.0.0')
+
+    def test_creates_endpoint_with_ipv6_url(self):
+        endpoint = self.creator.create_endpoint(
+            self.service_model, region_name='us-east-1',
+            endpoint_url='https://[100:0:2::61]:7480')
+        self.assertEqual(endpoint.host, 'https://[100:0:2::61]:7480')
+
+    def test_raises_error_with_invalid_url(self):
+        with pytest.raises(ValueError):
+            self.creator.create_endpoint(
+                self.service_model, region_name='us-east-1',
+                endpoint_url='https://*.aws.amazon.com/'
+            )
+
     def test_create_endpoint_with_default_timeout(self):
         self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
